@@ -17,6 +17,7 @@ class SAINT(LightningModule):
 
     :param categories: List with the number of unique values for each categorical column
     :param continuous: List of indices with continuous columns
+    :param dim_target: if categorical represent number of classes of the target otherwise is 1
     :param config: configuration of the model
     :param pretraining: boolean flag, if True it will be executed pretraining task
     """
@@ -24,6 +25,7 @@ class SAINT(LightningModule):
             self,
             categories: List[int],
             continuous: List[int],
+            dim_target: int,
             config: SaintConfig,
             pretraining: bool = False,
     ):
@@ -31,6 +33,7 @@ class SAINT(LightningModule):
         assert all(map(lambda n: n > 0, categories)), 'number of each category must be positive'
         self.config = config
         self.pretraining = pretraining
+        self.dim_target = dim_target
         self.num_categories = len(categories)
         self.num_unique_categories = sum(categories)
         self.num_continuous = len(continuous) if len(continuous) > 0 else 1
@@ -49,7 +52,7 @@ class SAINT(LightningModule):
         self._define_transformer()
         self._define_mlp(categories)
         self._define_projection_head()
-        self.mlpfory = SimpleMLP(self.config.network.embedding_size, 1000, categories[-1])
+        self.mlpfory = SimpleMLP(self.config.network.embedding_size, 1000, self.dim_target)
 
     def _define_transformer(self) -> None:
         """Instantiate the type of Transformed that will be used in SAINT"""
@@ -92,8 +95,6 @@ class SAINT(LightningModule):
         :param x_cont: contains the values for the continuous features
         """
         x_categ = x_categ + self.cat_mask_offset.type_as(x_categ)
-        # mask the target
-        x_categ[:, -1] = torch.zeros(x_categ.shape[0])
         x_categ_enc = self.embedding_categorical(x_categ)
         n1, n2 = x_cont.shape
         _, n3 = x_categ.shape
@@ -141,7 +142,7 @@ class SAINT(LightningModule):
         :param embed_categ: embeddings categorical features
         :param embed_cont: embeddings continuous features
         """
-        if self.config.pretrain.aug.get('cutmix'):
+        if self.config.pretrain.aug.cutmix:
             random_index = get_random_index(x_categ)
             x_categ_noised = cutmix(x_categ, random_index, self.config.pretrain.aug.cutmix.lam)
             x_cont_noised = cutmix(x_cont, random_index, self.config.pretrain.aug.cutmix.lam)
@@ -150,7 +151,7 @@ class SAINT(LightningModule):
             # if not apply cutmix the noised embeddings are equal to the original embeddings
             embed_categ_noised, embed_cont_noised = embed_categ, embed_cont
 
-        if self.config.pretrain.aug.get("mixup"):
+        if self.config.pretrain.aug.mixup:
             random_index = get_random_index(embed_categ_noised)
             embed_categ_noised = mixup(embed_categ_noised, random_index, lam=self.config.pretrain.aug.mixup.lam)
             embed_cont_noised = mixup(embed_cont_noised, random_index, lam=self.config.pretrain.aug.mixup.lam)
@@ -191,7 +192,7 @@ class SAINT(LightningModule):
         :param embed_categ_noised: embeddings categorical features after the augmentation
         :param embed_cont_noised: embeddings continuous features after the augmentation
         """
-        if self.config.pretrain.task.get("contrastive"):
+        if self.config.pretrain.task.contrastive:
             embed_tranformed, embed_transformed_noised = self._embeddings_contrastive(
                 embed_categ, embed_cont, embed_categ_noised, embed_cont_noised,
                 self.config.pretrain.task.contrastive.projhead_style)
@@ -204,7 +205,7 @@ class SAINT(LightningModule):
             loss_1 = f.cross_entropy(logits_1, targets)
             loss_2 = f.cross_entropy(logits_2, targets)
             return self.config.pretrain.task.contrastive.weight * (loss_1 + loss_2) / 2
-        elif self.config.pretrain.task.get("contrastive_sim"):
+        elif self.config.pretrain.task.contrastive_sim:
             # it apply the concept of simsiam we want that the embedding minimize the cosine similarity
             # the idea is that we want on the diagonal all 1, it means they are equal /because normalized)
             embed_tranformed, embed_transformed_noised = self._embeddings_contrastive(
@@ -213,7 +214,12 @@ class SAINT(LightningModule):
                 f.cosine_similarity(embed_tranformed, embed_transformed_noised).add_(-1).sum()
 
     def configure_optimizers(self):
-        return torch.optim.AdamW(self.parameters(), lr=0.02)
+        if self.pretraining:
+            lr = self.config.network.learning_rate_pretraining
+        else:
+            lr = self.config.network.learning_rate
+        optimizer = torch.optim.Adam(self.parameters(), lr=lr)
+        return optimizer
 
     def forward(self, x_categ: Tensor, x_cont: Tensor) -> Tensor:
         x_categ_enc, x_cont_enc = self._embed_data(x_categ, x_cont)
@@ -236,38 +242,61 @@ class SAINT(LightningModule):
                                                                                embed_categ, embed_cont)
         loss += self._pretraining_contrastive(embed_categ, embed_cont,
                                               embed_categ_noised, embed_cont_noised)
-        if self.config.pretrain.task.get("denoising"):
+        if self.config.pretrain.task.denoising:
             loss += self._pretraining_denoising(x_categ, x_cont, embed_categ_noised, embed_cont_noised)
         return loss
 
-    def _common_step(self, batch: Tuple[Tensor, Tensor]) -> Tensor:
-        x_categ, x_cont = batch
-        loss = Tensor([0])
+    def shared_step(self, batch: Tuple[Tensor, Tensor, Tensor]) -> Tensor:
+        """It define the commons step executed during training, validation and test
+
+        :param batch: Contains a batch of data
+        """
+        x_categ, x_cont, target = batch
         if self.pretraining:
-            loss += self.pretraining_step(x_categ, x_cont)
+            return self.pretraining_step(x_categ, x_cont)
         else:
             y_pred = self(x_categ, x_cont)
-            loss += f.cross_entropy(y_pred, x_categ[:, self.num_categories - 1])
-        return loss
+            if self.dim_target > 1:
+                return self._classification_loss(y_pred, target)
+            else:
+                return self._regression_loss(y_pred, target)
 
-    def training_step(self, batch: Tuple[Tensor, Tensor], batch_idx: int) -> Tensor:
-        loss = self._common_step(batch)
+    def training_step(self, batch: Tuple[Tensor, Tensor, Tensor], batch_idx: int) -> Tensor:
+        loss = self.shared_step(batch)
         self.log("train_loss", loss, on_step=False, on_epoch=True, prog_bar=True, logger=True)
         return loss
 
-    def validation_step(self, batch, batch_idx):
-        loss = self._common_step(batch)
+    def validation_step(self, batch: Tuple[Tensor, Tensor, Tensor], batch_idx: int) -> Tensor:
+        loss = self.shared_step(batch)
         self.log("validation_loss", loss, on_step=False, on_epoch=True, prog_bar=True, logger=True)
         return loss
 
-    def test_step(self, batch, batch_idx):
-        loss = self._common_step(batch)
+    def test_step(self, batch: Tuple[Tensor, Tensor, Tensor], batch_idx: int) -> Tensor:
+        loss = self.shared_step(batch)
         self.log("test_loss", loss, on_step=False, on_epoch=True, prog_bar=True, logger=True)
         return loss
 
-    def predict_step(self, batch: Tuple[Tensor, Tensor], batch_idx: int,
+    def predict_step(self, batch: Tuple[Tensor, Tensor, Tensor], batch_idx: int,
                      dataloader_idx: Optional[int] = None) -> Tensor:
-        x_categ, x_cont = batch
+        x_categ, x_cont, _ = batch
         y_pred = self(x_categ, x_cont)
         # return the class that has the greatest probability
-        return y_pred.argmax(1)
+        return y_pred.argmax(1) if self.dim_target > 1 else y_pred
+
+    @staticmethod
+    def _classification_loss(y_pred: Tensor, target: Tensor) -> Tensor:
+        """Loss function used in case of a classification problem
+
+        :param y_pred: Values predicted
+        :param target: Values to predict
+        """
+        return f.cross_entropy(y_pred, target)
+
+    @staticmethod
+    def _regression_loss(y_pred: Tensor, target: Tensor) -> Tensor:
+        """Loss function used in case of regression problem
+
+        :param y_pred: Values predicted
+        :param target: Values to predict
+        """
+        return f.mse_loss(y_pred, target)
